@@ -51,23 +51,23 @@ string nowTime()
   return t;
 }
 
-u_int16_t getShort(const unsigned char *p)
+uint16_t getShort(const unsigned char *p)
 {
   return p[0] * 256 + p[1];
 }
 
 
-u_int16_t getShort(const char *p)
+uint16_t getShort(const char *p)
 {
   return getShort((const unsigned char *)p);
 }
 
-u_int32_t getLong(const unsigned char* p)
+uint32_t getLong(const unsigned char* p)
 {
   return (p[0]<<24) + (p[1]<<16) + (p[2]<<8) + p[3];
 }
 
-u_int32_t getLong(const char* p)
+uint32_t getLong(const char* p)
 {
   return getLong((unsigned char *)p);
 }
@@ -119,13 +119,13 @@ bool endsOn(const string &domain, const string &suffix)
 
 int sendData(const char *buffer, int replen, int outsock)
 {
-  u_int16_t nlen=htons(replen);
+  uint16_t nlen=htons(replen);
   Utility::iovec iov[2];
   iov[0].iov_base=(char*)&nlen;
   iov[0].iov_len=2;
   iov[1].iov_base=(char*)buffer;
   iov[1].iov_len=replen;
-  int ret=Utility::writev(outsock,iov,2);
+  int ret=writev(outsock,iov,2);
 
   if(ret<0) {
     return -1;
@@ -150,13 +150,13 @@ void parseService(const string &descr, ServiceTuple &st)
 }
 
 
-int waitForData(int fd, int seconds)
+int waitForData(int fd, int seconds, int useconds)
 {
   struct timeval tv;
   int ret;
 
   tv.tv_sec   = seconds;
-  tv.tv_usec  = 0;
+  tv.tv_usec  = useconds;
 
   fd_set readfds;
   FD_ZERO( &readfds );
@@ -281,7 +281,7 @@ void cleanSlashes(string &str)
 }
 
 
-bool IpToU32(const string &str, u_int32_t *ip)
+bool IpToU32(const string &str, uint32_t *ip)
 {
   if(str.empty()) {
     *ip=0;
@@ -289,7 +289,7 @@ bool IpToU32(const string &str, u_int32_t *ip)
   }
   
   struct in_addr inp;
-  if(Utility::inet_aton(str.c_str(), &inp)) {
+  if(inet_aton(str.c_str(), &inp)) {
     *ip=inp.s_addr;
     return true;
   }
@@ -307,7 +307,7 @@ const string sockAddrToString(struct sockaddr_in *remote, Utility::socklen_t soc
   else {
     char tmp[128];
     
-    if(!Utility::inet_ntop(AF_INET6, ( const char * ) &((struct sockaddr_in6 *)remote)->sin6_addr, tmp, sizeof(tmp)))
+    if(!inet_ntop(AF_INET6, ( const char * ) &((struct sockaddr_in6 *)remote)->sin6_addr, tmp, sizeof(tmp)))
       return "IPv6 untranslateable";
 
     return tmp;
@@ -316,3 +316,18 @@ const string sockAddrToString(struct sockaddr_in *remote, Utility::socklen_t soc
 
   return "untranslateable";
 }
+
+string makeHexDump(const string& str)
+{
+  char tmp[5];
+  string ret;
+  ret.reserve((int)(str.size()*2.2));
+
+  for(string::size_type n=0;n<str.size();++n) {
+    sprintf(tmp,"%02x ", (unsigned char)str[n]);
+    ret+=tmp;
+  }
+  return ret;
+}
+
+
