@@ -237,7 +237,8 @@ template<class Key, class Val>void MTasker<Key,Val>::makeThread(tfunc_t *start,
   
   uc->uc_stack.ss_size = d_stacksize;
 #ifdef SOLARIS
-  makecontext (uc, (void (*)(...))threadWrapper, 4, this, start, d_maxtid, val);
+  uc->uc_stack.ss_sp = (void*)(((char*)uc->uc_stack.ss_sp)+d_stacksize);
+  makecontext (uc,(void (*)(...))threadWrapper, 5, this, start, d_maxtid, val);
 #else
   makecontext (uc, (void (*)(void))threadWrapper, 4, this, start, d_maxtid, val);
 #endif
@@ -278,7 +279,7 @@ template<class Key, class Val>bool MTasker<Key,Val>::schedule()
   }
   if(!d_waiters.empty()) {
     time_t now=time(0);
-    for(typename waiters_t::const_iterator i=d_waiters.begin();i!=d_waiters.end();++i) {
+    for(typename waiters_t::iterator i=d_waiters.begin();i!=d_waiters.end(); ) {
       if(i->second.ttd && i->second.ttd<now) {
 	d_waitstatus=TimeOut;
 	if(swapcontext(&d_kernel,i->second.context)) { // swaps back to the above point 'A'
@@ -286,8 +287,10 @@ template<class Key, class Val>bool MTasker<Key,Val>::schedule()
 	  exit(EXIT_FAILURE);
 	}
 	delete i->second.context;              
-	d_waiters.erase(i->first);                  // removes the waitpoint 
+	d_waiters.erase(i++);                  // removes the waitpoint 
       }
+      else 
+	++i;
     }
   }
   return false;
