@@ -40,7 +40,11 @@
 #include <boost/tuple/tuple.hpp>
 #include <boost/tuple/tuple_comparison.hpp>
 #include <boost/shared_array.hpp>
-
+#include <boost/lexical_cast.hpp>
+#include "dnsparser.hh"
+#include "dnswriter.hh"
+#include "dnsrecords.hh"
+#include "zoneparser-tng.hh"
 using namespace boost;
 
 #include "recursor_cache.hh"
@@ -54,6 +58,34 @@ MemRecursorCache RC;
 
 string s_programname="pdns_recursor";
 
+struct DNSComboWriter {
+  DNSComboWriter(const char* data, uint16_t len) : d_mdp(data, len), d_tcp(false), d_socket(-1)
+  {}
+  MOADNSParser d_mdp;
+  void setRemote(struct sockaddr* sa, socklen_t len)
+  {
+    memcpy((void *)d_remote, (void *)sa, len);
+    d_socklen=len;
+  }
+
+  void setSocket(int sock)
+  {
+    d_socket=sock;
+  }
+
+  string getRemote() const
+  {
+    return sockAddrToString((struct sockaddr_in *)d_remote, d_socklen);
+  }
+
+
+  char d_remote[sizeof(sockaddr_in6)];
+  socklen_t d_socklen;
+  bool d_tcp;
+  int d_socket;
+};
+
+
 #ifndef WIN32
 #ifndef __FreeBSD__
 extern "C" {
@@ -88,7 +120,7 @@ struct PacketID
   PacketID() : sock(0), inNeeded(0), outPos(0)
   {}
 
-  u_int16_t id;  // wait for a specific id/remote paie
+  uint16_t id;  // wait for a specific id/remote paie
   struct sockaddr_in remote;  // this is the remote
 
   Socket* sock;  // or wait for an event on a TCP fd
@@ -161,10 +193,10 @@ int arecvfrom(char *data, int len, int flags, struct sockaddr *toaddr, Utility::
 {
   PacketID pident;
   pident.id=id;
-  memcpy(&pident.remote,toaddr,sizeof(pident.remote));
+  memcpy(&pident.remote, toaddr, sizeof(pident.remote));
 
   string packet;
-  int ret=MT->waitEvent(pident,&packet,1);
+  int ret=MT->waitEvent(pident, &packet, 1);
   if(ret > 0) {
     *d_len=packet.size();
     memcpy(data,packet.c_str(),min(len,*d_len));
@@ -187,86 +219,125 @@ static void writePid(void)
 void primeHints(void)
 {
   // prime root cache
-
-  static char*ips[]={"198.41.0.4", "192.228.79.201", "192.33.4.12", "128.8.10.90", "192.203.230.10", "192.5.5.241", "192.112.36.4", "128.63.2.53", 
-		     "192.36.148.17","192.58.128.30", "193.0.14.129", "198.32.64.12", "202.12.27.33"};
-  DNSResourceRecord arr, nsrr;
-  arr.qtype=QType::A;
-  arr.ttl=time(0)+3600000;
-  nsrr.qtype=QType::NS;
-  nsrr.ttl=time(0)+3600000;
-  
   set<DNSResourceRecord>nsset;
-  for(char c='a';c<='m';++c) {
-    static char templ[40];
-    strncpy(templ,"a.root-servers.net", sizeof(templ) - 1);
-    *templ=c;
-    arr.qname=nsrr.content=templ;
-    arr.content=ips[c-'a'];
-    set<DNSResourceRecord>aset;
-    aset.insert(arr);
-    RC.replace(string(templ),QType(QType::A),aset);
 
-    nsset.insert(nsrr);
+  if(arg()["hint-file"].empty()) {
+    static char*ips[]={"198.41.0.4", "192.228.79.201", "192.33.4.12", "128.8.10.90", "192.203.230.10", "192.5.5.241", "192.112.36.4", "128.63.2.53", 
+		       "192.36.148.17","192.58.128.30", "193.0.14.129", "198.32.64.12", "202.12.27.33"};
+    DNSResourceRecord arr, nsrr;
+    arr.qtype=QType::A;
+    arr.ttl=time(0)+3600000;
+    nsrr.qtype=QType::NS;
+    nsrr.ttl=time(0)+3600000;
+    
+
+    for(char c='a';c<='m';++c) {
+      static char templ[40];
+      strncpy(templ,"a.root-servers.net", sizeof(templ) - 1);
+      *templ=c;
+      arr.qname=nsrr.content=templ;
+      arr.content=ips[c-'a'];
+      set<DNSResourceRecord> aset;
+      aset.insert(arr);
+      RC.replace(string(templ), QType(QType::A), aset);
+      
+      nsset.insert(nsrr);
+    }
   }
-  RC.replace("",QType(QType::NS),nsset); // and stuff in the cache
+  else {
+    ZoneParserTNG zpt(arg()["hint-file"]);
+    DNSResourceRecord rr;
+    set<DNSResourceRecord> aset;
+
+    while(zpt.get(rr)) {
+      //      cout<<"'"<<rr.qname<<"' "<<rr.qtype.getName()<<" '"<<rr.content<<"'\n";
+
+      rr.ttl+=time(0);
+      if(rr.qtype.getCode()==QType::A) {
+	set<DNSResourceRecord> aset;
+	aset.insert(rr);
+	RC.replace(rr.qname, QType(QType::A), aset);
+      }
+      if(rr.qtype.getCode()==QType::NS) {
+	nsset.insert(rr);
+      }
+    }
+  }
+  RC.replace("", QType(QType::NS), nsset); // and stuff in the cache
 }
 
 void startDoResolve(void *p)
 {
   try {
     bool quiet=arg().mustDo("quiet");
-    DNSPacket P=*(DNSPacket *)p;
+    DNSComboWriter* dc=(DNSComboWriter *)p;
 
-    delete (DNSPacket *)p;
+    uint16_t maxudpsize=512;
+    MOADNSParser::EDNSOpts edo;
+    if(dc->d_mdp.getEDNSOpts(&edo)) {
+      maxudpsize=edo.d_packetsize;
+    }
+
+    vector<DNSResourceRecord> ret;
     
-    vector<DNSResourceRecord>ret;
-    DNSPacket *R=P.replyPacket();
-    R->setA(false);
-    R->setRA(true);
+    vector<uint8_t> packet;
+    DNSPacketWriter pw(packet, dc->d_mdp.d_qname, dc->d_mdp.d_qtype, dc->d_mdp.d_qclass);
+
+    pw.getHeader()->aa=0;
+    pw.getHeader()->ra=1;
+    pw.getHeader()->qr=1;
+    pw.getHeader()->id=dc->d_mdp.d_header.id;
+    pw.getHeader()->rd=dc->d_mdp.d_header.rd;
+
     //    MT->setTitle("udp question for "+P.qdomain+"|"+P.qtype.getName());
     SyncRes sr;
     if(!quiet)
-      L<<Logger::Error<<"["<<MT->getTid()<<"] " << (R->d_tcp ? "TCP " : "") << "question for '"<<P.qdomain<<"|"<<P.qtype.getName()<<"' from "<<P.getRemote()<<endl;
+      L<<Logger::Error<<"["<<MT->getTid()<<"] " << (dc->d_tcp ? "TCP " : "") << "question for '"<<dc->d_mdp.d_qname<<"|"
+       <<DNSRecordContent::NumberToType(dc->d_mdp.d_qtype)<<"' from "<<dc->getRemote()<<endl;
 
     sr.setId(MT->getTid());
-    if(!P.d.rd)
+    if(!dc->d_mdp.d_header.rd)
       sr.setCacheOnly();
 
-    int res=sr.beginResolve(P.qdomain, P.qtype, ret);
+    int res=sr.beginResolve(dc->d_mdp.d_qname, QType(dc->d_mdp.d_qtype), ret);
     if(res<0)
-      R->setRcode(RCode::ServFail);
+      pw.getHeader()->rcode=RCode::ServFail;
     else {
-      R->setRcode(res);
-      for(vector<DNSResourceRecord>::const_iterator i=ret.begin();i!=ret.end();++i)
-	R->addRecord(*i);
-    }
-
-    const char *buffer=R->getData();
-
-    if(!R->d_tcp) {
-      if(R->len > 512) {
-	R->truncate(512);
+      pw.getHeader()->rcode=res;
+      if(ret.size()) {
+	for(vector<DNSResourceRecord>::const_iterator i=ret.begin();i!=ret.end();++i) {
+	  pw.startRecord(i->qname, i->qtype.getCode(), i->ttl, 1, (DNSPacketWriter::Place)i->d_place);
+	  shared_ptr<DNSRecordContent> drc(DNSRecordContent::mastermake(i->qtype.getCode(), 1, i->content));  
+	  drc->toPacket(pw);
+	  if(!dc->d_tcp && pw.size() > maxudpsize) {
+	    pw.rollback();
+	    pw.getHeader()->tc=1;
+	    goto sendit; // need to jump over pw.commit
+	  }
+	}
+	pw.commit();
       }
-
-      sendto(R->getSocket(),buffer,R->len,0,(struct sockaddr *)(R->remote),R->d_socklen);
+    }
+  sendit:;
+    if(!dc->d_tcp) {
+      sendto(dc->d_socket, &*packet.begin(), packet.size(), 0, (struct sockaddr *)(dc->d_remote), dc->d_socklen);
     }
     else {
       char buf[2];
-      buf[0]=R->len/256;
-      buf[1]=R->len%256;
+      buf[0]=packet.size()/256;
+      buf[1]=packet.size()%256;
 
       struct iovec iov[2];
 
-      iov[0].iov_base=(void*)buf;    iov[0].iov_len=2;
-      iov[1].iov_base=(void*)buffer; iov[1].iov_len = R->len;
+      iov[0].iov_base=(void*)buf;              iov[0].iov_len=2;
+      iov[1].iov_base=(void*)&*packet.begin(); iov[1].iov_len = packet.size();
 
-      int ret=writev(R->getSocket(), iov, 2);
+      int ret=writev(dc->d_socket, iov, 2);
 
       if(ret <= 0 ) 
-	L<<Logger::Error<<"Error writing TCP answer to "<<P.getRemote()<<": "<< (ret ? strerror(errno) : "EOF") <<endl;
-      else if(ret != 2 + R->len)
-	L<<Logger::Error<<"Oops, partial answer sent to "<<P.getRemote()<<" - probably would have trouble receiving our answer anyhow (size="<<R->len<<")"<<endl;
+	L<<Logger::Error<<"Error writing TCP answer to "<<dc->getRemote()<<": "<< (ret ? strerror(errno) : "EOF") <<endl;
+      else if((unsigned int)ret != 2 + packet.size())
+	L<<Logger::Error<<"Oops, partial answer sent to "<<dc->getRemote()<<" - probably would have trouble receiving our answer anyhow (size="<<packet.size()<<")"<<endl;
 
       //      if(write(R->getSocket(),buf,2)!=2 || write(R->getSocket(),buffer,R->len)!=R->len)
       //  XXX FIXME write this writev fallback otherwise
@@ -274,18 +345,20 @@ void startDoResolve(void *p)
 
     //    MT->setTitle("DONE! udp question for "+P.qdomain+"|"+P.qtype.getName());
     if(!quiet) {
-      L<<Logger::Error<<"["<<MT->getTid()<<"] answer to "<<(P.d.rd?"":"non-rd ")<<"question '"<<P.qdomain<<"|"<<P.qtype.getName();
-      L<<"': "<<ntohs(R->d.ancount)<<" answers, "<<ntohs(R->d.arcount)<<" additional, took "<<sr.d_outqueries<<" packets, "<<
+      L<<Logger::Error<<"["<<MT->getTid()<<"] answer to "<<(dc->d_mdp.d_header.rd?"":"non-rd ")<<"question '"<<dc->d_mdp.d_qname<<"|"<<DNSRecordContent::NumberToType(dc->d_mdp.d_qtype);
+      L<<"': "<<ntohs(pw.getHeader()->ancount)<<" answers, "<<ntohs(pw.getHeader()->arcount)<<" additional, took "<<sr.d_outqueries<<" packets, "<<
 	sr.d_throttledqueries<<" throttled, "<<sr.d_timeouts<<" timeouts, "<<sr.d_tcpoutqueries<<" tcp connections, rcode="<<res<<endl;
     }
     
     sr.d_outqueries ? RC.cacheMisses++ : RC.cacheHits++; 
-
-    delete R;
+    delete dc;
   }
   catch(AhuException &ae) {
     L<<Logger::Error<<"startDoResolve problem: "<<ae.reason<<endl;
   }
+  catch(exception& e) {
+    L<<Logger::Error<<"STL error: "<<e.what()<<endl;
+  }
   catch(...) {
     L<<Logger::Error<<"Any other exception in a resolver context"<<endl;
   }
@@ -307,7 +380,7 @@ void makeClientSocket()
 
   int tries=10;
   while(--tries) {
-    u_int16_t port=10000+Utility::random()%10000;
+    uint16_t port=10000+Utility::random()%10000;
     sin.sin_port = htons(port); 
     
     if (bind(d_clientsock, (struct sockaddr *)&sin, sizeof(sin)) >= 0) 
@@ -407,7 +480,6 @@ void daemonize(void)
   close(0);
   close(1);
   close(2);
-
 }
 #endif
 
@@ -424,8 +496,6 @@ void usr2Handler(int)
   SyncRes::setLog(true);
 }
 
-
-
 void doStats(void)
 {
   if(qcounter) {
@@ -456,7 +526,7 @@ void houseKeeping(void *)
   }
   if(now -last_rootupdate>7200) {
     SyncRes sr;
-    vector<DNSResourceRecord>ret;
+    vector<DNSResourceRecord> ret;
 
     sr.setNoCache();
     int res=sr.beginResolve("", QType(QType::NS), ret);
@@ -480,8 +550,40 @@ struct TCPConnection
   time_t startTime;
 };
 
+#if 0
+
+#include <execinfo.h>
+
+static void maketrace()
+{
+  void *array[20]; //only care about last 17 functions (3 taken with tracing support)
+  size_t size;
+  char **strings;
+  size_t i;
+
+  size = backtrace (array, 20);
+  strings = backtrace_symbols (array, size); //Need -rdynamic gcc (linker) flag for this to work
+
+  for (i = 0; i < size; i++) //skip useless functions
+    cout<<strings[i]<<"\n";
+  cout<<"--"<<endl;
+}
+
+extern "C" {
+int gettimeofday (struct timeval *__restrict __tv,
+		  __timezone_ptr_t __tz)
+{
+  maketrace();
+  return 0;
+}
+
+}
+#endif 
+
 int main(int argc, char **argv) 
 {
+  reportBasicTypes();
+
   int ret = EXIT_SUCCESS;
 #ifdef WIN32
     WSADATA wsaData;
@@ -501,13 +603,14 @@ int main(int argc, char **argv)
     arg().set("chroot","switch to chroot jail")="";
     arg().set("setgid","If set, change group id to this gid for more security")="";
     arg().set("setuid","If set, change user id to this uid for more security")="";
-    arg().set("quiet","Suppress logging of questions and answers")="off";
+    arg().set("quiet","Suppress logging of questions and answers")="true";
     arg().set("config-dir","Location of configuration directory (recursor.conf)")=SYSCONFDIR;
     arg().set("socket-dir","Where the controlsocket will live")=LOCALSTATEDIR;
     arg().set("delegation-only","Which domains we only accept delegations from")="";
     arg().set("query-local-address","Source IP address for sending queries")="0.0.0.0";
     arg().set("client-tcp-timeout","Timeout in seconds when talking to TCP clients")="2";
     arg().set("max-tcp-clients","Maximum number of simultaneous TCP clients")="128";
+    arg().set("hint-file", "If set, load root hints from this file")="";
 
     arg().setCmd("help","Provide a helpful message");
     L.toConsole(Logger::Warning);
@@ -542,8 +645,11 @@ int main(int argc, char **argv)
     "according to the terms of the GPL version 2."<<endl;
 
 
-    if(arg().mustDo("trace"))
+  if(arg().mustDo("trace")) {
       SyncRes::setLog(true);
+      arg().set("quiet")="no";
+      
+  }
     
     makeClientSocket();
     makeUDPServerSockets();
@@ -600,7 +706,6 @@ int main(int argc, char **argv)
 
       Utility::socklen_t addrlen=sizeof(fromaddr);
       int d_len;
-      DNSPacket P;
       
       struct timeval tv;
       tv.tv_sec=0;
@@ -663,22 +768,25 @@ int main(int argc, char **argv)
 	d_len=recvfrom(d_clientsock, data, sizeof(data), 0, (sockaddr *)&fromaddr, &addrlen);    
 	if(d_len<0) 
 	  continue;
-	
-	P.setRemote((struct sockaddr *)&fromaddr, addrlen);
-	if(P.parse(data,d_len)<0) {
-	  L<<Logger::Error<<"Unparseable packet from remote server "<<P.getRemote()<<endl;
-	}
-	else { 
-	  if(P.d.qr) {
+
+	try {
+	  DNSComboWriter dc(data, d_len);
+	  dc.setRemote((struct sockaddr *)&fromaddr, addrlen);
+
+	  if(dc.d_mdp.d_header.qr) {
 	    pident.remote=fromaddr;
-	    pident.id=P.d.id;
+	    pident.id=dc.d_mdp.d_header.id;
 	    string packet;
-	    packet.assign(data,d_len);
-	    MT->sendEvent(pident,&packet);
+	    packet.assign(data, d_len);
+	    MT->sendEvent(pident, &packet);
 	  }
 	  else 
-	    L<<Logger::Warning<<"Ignoring question on outgoing socket from "<<P.getRemote()<<endl;
+	    L<<Logger::Warning<<"Ignoring question on outgoing socket from "<<dc.getRemote()<<endl;
 	}
+	catch(MOADNSException& mde) {
+	  L<<Logger::Error<<"Unparseable packet from remote server "<< sockAddrToString((struct sockaddr_in*) &fromaddr, addrlen) <<": "<<mde.what()<<endl;
+	}
+
       }
       
       for(vector<int>::const_iterator i=d_udpserversocks.begin(); i!=d_udpserversocks.end(); ++i) {
@@ -686,19 +794,18 @@ int main(int argc, char **argv)
 	  d_len=recvfrom(*i, data, sizeof(data), 0, (sockaddr *)&fromaddr, &addrlen);    
 	  if(d_len<0) 
 	    continue;
-	  P.setRemote((struct sockaddr *)&fromaddr, addrlen);
-	  if(P.parse(data,d_len)<0) {
-	    L<<Logger::Error<<"Unparseable packet from remote client "<<P.getRemote()<<endl;
-	  }
-	  else { 
-	    if(P.d.qr)
-	      L<<Logger::Error<<"Ignoring answer on server socket!"<<endl;
-	    else {
-	      ++qcounter;
-	      P.setSocket(*i);
-	      P.d_tcp=false;
-	      MT->makeThread(startDoResolve,(void*)new DNSPacket(P), "udp");
-	    }
+
+	  DNSComboWriter* dc = new DNSComboWriter(data, d_len);
+
+	  dc->setRemote((struct sockaddr *)&fromaddr, addrlen);
+
+	  if(dc->d_mdp.d_header.qr)
+	    L<<Logger::Error<<"Ignoring answer on server socket!"<<endl;
+	  else {
+	    ++qcounter;
+	    dc->setSocket(*i);
+	    dc->d_tcp=false;
+	    MT->makeThread(startDoResolve, (void*) dc, "udp");
 	  }
 	}
       }
@@ -823,23 +930,25 @@ int main(int argc, char **argv)
 	    i->bytesread+=bytes;
 	    if(i->bytesread==i->qlen) {
 	      i->state=TCPConnection::BYTE0;
-
-	      if(P.parse(i->data,i->qlen)<0) {
-		L<<Logger::Error<<"Unparseable packet from remote client "<<P.getRemote()<<endl;
+	      DNSComboWriter* dc=0;
+	      try {
+		dc=new DNSComboWriter(i->data, i->qlen);
+	      }
+	      catch(MOADNSException &mde) {
+		L<<Logger::Error<<"Unparseable packet from remote client "<<sockAddrToString(&i->remote,sizeof(i->remote))<<endl;
 		close(i->fd);
 		tcpconnections.erase(i);
 		break;
 	      }
-	      else { 
-		P.setSocket(i->fd);
-		P.d_tcp=true;
-		P.setRemote((struct sockaddr *)&i->remote,sizeof(i->remote));
-		if(P.d.qr)
-		  L<<Logger::Error<<"Ignoring answer on server socket!"<<endl;
-		else {
-		  ++qcounter;
-		  MT->makeThread(startDoResolve,(void*)new DNSPacket(P), "tcp");
-		}
+
+	      dc->setSocket(i->fd);
+	      dc->d_tcp=true;
+	      dc->setRemote((struct sockaddr *)&i->remote,sizeof(i->remote));
+	      if(dc->d_mdp.d_header.qr)
+		L<<Logger::Error<<"Ignoring answer on server socket!"<<endl;
+	      else {
+		++qcounter;
+		MT->makeThread(startDoResolve, dc, "tcp");
 	      }
 	    }
 	  }
