@@ -1,6 +1,6 @@
 /*
     PowerDNS Versatile Database Driven Nameserver
-    Copyright (C) 2005  PowerDNS.COM BV
+    Copyright (C) 2003 - 2005  PowerDNS.COM BV
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License version 2 as published 
@@ -32,6 +32,7 @@
 #include "arguments.hh"
 #include "lwres.hh"
 #include "recursor_cache.hh"
+#include "dnsparser.hh"
 
 extern MemRecursorCache RC;
 
@@ -44,7 +45,7 @@ unsigned int SyncRes::s_throttledqueries;
 unsigned int SyncRes::s_nodelegated;
 bool SyncRes::s_log;
 
-#define LOG if(s_log)L<<Logger::Warning
+#define LOG if(s_log) L<<Logger::Warning
 
 Throttle<string> SyncRes::s_throttle;
 
@@ -117,22 +118,25 @@ void SyncRes::getBestNSFromCache(const string &qname, set<DNSResourceRecord>&bes
   do {
     LOG<<prefix<<qname<<": Checking if we have NS in cache for '"<<subdomain<<"'"<<endl;
     set<DNSResourceRecord>ns;
-    if(RC.get(d_now, subdomain,QType(QType::NS),&ns)>0) {
+    if(RC.get(d_now.tv_sec, subdomain, QType(QType::NS), &ns)>0) {
 
       for(set<DNSResourceRecord>::const_iterator k=ns.begin();k!=ns.end();++k) {
-	if(k->ttl > (unsigned int)d_now ) { 
+	if(k->ttl > (unsigned int)d_now.tv_sec ) { 
 	  set<DNSResourceRecord>aset;
-	  if(!endsOn(k->content,subdomain) || RC.get(d_now, k->content,QType(QType::A),&aset) > 5) {
-	    bestns.insert(*k);
-	    LOG<<prefix<<qname<<": NS (with ip, or non-glue) in cache for '"<<subdomain<<"' -> '"<<k->content<<"'"<<endl;
-	    LOG<<prefix<<qname<<": within bailiwick: "<<endsOn(k->content,subdomain);
+
+	  DNSResourceRecord rr=*k;
+	  rr.content=toLowerCanonic(k->content);
+	  if(!endsOn(rr.content,subdomain) || RC.get(d_now.tv_sec, rr.content ,QType(QType::A),&aset) > 5) {
+	    bestns.insert(rr);
+	    LOG<<prefix<<qname<<": NS (with ip, or non-glue) in cache for '"<<subdomain<<"' -> '"<<rr.content<<"'"<<endl;
+	    LOG<<prefix<<qname<<": within bailiwick: "<<endsOn(rr.content,subdomain);
 	    if(!aset.empty())
-	      L<<", in cache, ttl="<<(unsigned int)(((time_t)aset.begin()->ttl- d_now ))<<endl;
+	      L<<", in cache, ttl="<<(unsigned int)(((time_t)aset.begin()->ttl- d_now.tv_sec ))<<endl;
 	    else
 	      L<<", not in cache"<<endl;
 	  }
 	  else
-	    LOG<<prefix<<qname<<": NS in cache for '"<<subdomain<<"', but needs glue ("<<k->content<<") which we miss or is expired"<<endl;
+	    LOG<<prefix<<qname<<": NS in cache for '"<<subdomain<<"', but needs glue ("<<toLowerCanonic(k->content)<<") which we miss or is expired"<<endl;
 	}
       }
       if(!bestns.empty()) {
@@ -172,7 +176,7 @@ string SyncRes::getBestNSNamesFromCache(const string &qname,set<string>& nsset,
 
 bool SyncRes::doCNAMECacheCheck(const string &qname, const QType &qtype, vector<DNSResourceRecord>&ret, int depth, int &res)
 {
-  string prefix(d_prefix), tuple=toLower(qname)+"|CNAME";
+  string prefix(d_prefix), tuple=toLowerCanonic(qname)+"|CNAME";
   prefix.append(depth, ' ');
 
   if(depth>10) {
@@ -183,18 +187,20 @@ bool SyncRes::doCNAMECacheCheck(const string &qname, const QType &qtype, vector<
   
   LOG<<prefix<<qname<<": Looking for CNAME cache hit of '"<<tuple<<"'"<<endl;
   set<DNSResourceRecord> cset;
-  if(RC.get(d_now, qname,QType(QType::CNAME),&cset) > 0) {
+  if(RC.get(d_now.tv_sec, qname,QType(QType::CNAME),&cset) > 0) {
 
     for(set<DNSResourceRecord>::const_iterator j=cset.begin();j!=cset.end();++j) {
-      if(j->ttl>(unsigned int) d_now) {
+      if(j->ttl>(unsigned int) d_now.tv_sec) {
 	LOG<<prefix<<qname<<": Found cache CNAME hit for '"<<tuple<<"' to '"<<j->content<<"'"<<endl;    
 	DNSResourceRecord rr=*j;
-	rr.ttl-=d_now;
+	rr.ttl-=d_now.tv_sec;
 	ret.push_back(rr);
-	if(!(qtype==QType(QType::CNAME))) {// perhaps they really wanted a CNAME!
+	if(!(qtype==QType(QType::CNAME))) { // perhaps they really wanted a CNAME!
 	  set<GetBestNSAnswer>beenthere;
-	  res=doResolve(j->content, qtype, ret, depth+1, beenthere);
+	  res=doResolve(toLowerCanonic(j->content), qtype, ret, depth+1, beenthere);
 	}
+	else
+	  res=0;
 	return true;
       }
     }
@@ -203,9 +209,6 @@ bool SyncRes::doCNAMECacheCheck(const string &qname, const QType &qtype, vector<
   return false;
 }
 
-
-
-
 bool SyncRes::doCacheCheck(const string &qname, const QType &qtype, vector<DNSResourceRecord>&ret, int depth, int &res)
 {
   bool giveNegative=false;
@@ -214,13 +217,13 @@ bool SyncRes::doCacheCheck(const string &qname, const QType &qtype, vector<DNSRe
 
   string sqname(qname);
   QType sqt(qtype);
-  u_int32_t sttl=0;
+  uint32_t sttl=0;
 
   if(s_negcache.count(toLower(qname))) {
     res=0;
     map<string,NegCacheEntry>::const_iterator ni=s_negcache.find(toLower(qname));
-    if(d_now < ni->second.ttd) {
-      sttl=ni->second.ttd - d_now;
+    if(d_now.tv_sec < ni->second.ttd) {
+      sttl=ni->second.ttd - d_now.tv_sec;
       LOG<<prefix<<qname<<": Entire record '"<<toLower(qname)<<"', is negatively cached for another "<<sttl<<" seconds"<<endl;
       res=RCode::NXDomain; 
       giveNegative=true;
@@ -241,8 +244,8 @@ bool SyncRes::doCacheCheck(const string &qname, const QType &qtype, vector<DNSRe
     res=0;
     map<string,NegCacheEntry>::const_iterator ni=s_negcache.find(tuple);
     if(ni!=s_negcache.end()) {
-      if(d_now < ni->second.ttd) {
-	sttl=ni->second.ttd - d_now;
+      if(d_now.tv_sec < ni->second.ttd) {
+	sttl=ni->second.ttd - d_now.tv_sec;
 	LOG<<prefix<<qname<<": "<<qtype.getName()<<" is negatively cached for another "<<sttl<<" seconds"<<endl;
 	res=RCode::NoError; // only this record doesn't exist
 	giveNegative=true;
@@ -258,13 +261,13 @@ bool SyncRes::doCacheCheck(const string &qname, const QType &qtype, vector<DNSRe
 
   set<DNSResourceRecord> cset;
   bool found=false, expired=false;
-  if(RC.get(d_now, sqname,sqt,&cset)>0) {
+  if(RC.get(d_now.tv_sec, sqname,sqt,&cset)>0) {
     LOG<<prefix<<qname<<": Found cache hit for "<<sqt.getName()<<": ";
     for(set<DNSResourceRecord>::const_iterator j=cset.begin();j!=cset.end();++j) {
       LOG<<j->content;
-      if(j->ttl>(unsigned int) d_now) {
+      if(j->ttl>(unsigned int) d_now.tv_sec) {
 	DNSResourceRecord rr=*j;
-	rr.ttl-=d_now;
+	rr.ttl-=d_now.tv_sec;
 	if(giveNegative) {
 	  rr.d_place=DNSResourceRecord::AUTHORITY;
 	  rr.ttl=sttl;
@@ -321,7 +324,8 @@ inline vector<string> SyncRes::shuffle(set<string> &nameservers, const string &p
 
   for(set<string>::const_iterator i=nameservers.begin();i!=nameservers.end();++i) {
     rnameservers.push_back(*i);
-    speeds[*i]=nsSpeeds[toLower(*i)].get();
+    DecayingEwma& temp=nsSpeeds[toLower(*i)];
+    speeds[*i]=temp.get(&d_now);
   }
   random_shuffle(rnameservers.begin(),rnameservers.end());
   stable_sort(rnameservers.begin(),rnameservers.end(),speedOrder(speeds));
@@ -338,7 +342,6 @@ inline vector<string> SyncRes::shuffle(set<string> &nameservers, const string &p
     }
     L<<endl;
   }
-
   return rnameservers;
 }
 
@@ -377,7 +380,7 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
 
       bool doTCP=false;
 
-      if(s_throttle.shouldThrottle(d_now, remoteIP+"|"+qname+"|"+qtype.getName())) {
+      if(s_throttle.shouldThrottle(d_now.tv_sec, remoteIP+"|"+qname+"|"+qtype.getName())) {
 	LOG<<prefix<<qname<<": query throttled "<<endl;
 	s_throttledqueries++;
 	d_throttledqueries++;
@@ -402,12 +405,12 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
 	  else
 	    LOG<<prefix<<qname<<": error resolving"<<endl;
 
-	  nsSpeeds[toLower(*tns)].submit(1000000); // 1 sec
+	  nsSpeeds[toLower(*tns)].submit(1000000, &d_now); // 1 sec
 	  
-	  s_throttle.throttle(d_now, remoteIP+"|"+qname+"|"+qtype.getName(),20,5);
+	  s_throttle.throttle(d_now.tv_sec, remoteIP+"|"+qname+"|"+qtype.getName(),20,5);
 	  continue;
 	}
-	d_now=time(0);
+	gettimeofday(&d_now, 0);
       }
 
       result=d_lwr.result();
@@ -424,11 +427,11 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
 
       if(d_lwr.d_rcode==RCode::ServFail) {
 	LOG<<prefix<<qname<<": "<<*tns<<" returned a ServFail, trying sibling NS"<<endl;
-	s_throttle.throttle(d_now,remoteIP+"|"+qname+"|"+qtype.getName(),60,3);
+	s_throttle.throttle(d_now.tv_sec,remoteIP+"|"+qname+"|"+qtype.getName(),60,3);
 	continue;
       }
       LOG<<prefix<<qname<<": Got "<<result.size()<<" answers from "<<*tns<<" ("<<remoteIP<<"), rcode="<<d_lwr.d_rcode<<", in "<<d_lwr.d_usec/1000<<"ms"<<endl;
-      nsSpeeds[toLower(*tns)].submit(d_lwr.d_usec);
+      nsSpeeds[toLower(*tns)].submit(d_lwr.d_usec, &d_now);
 
       map<string,set<DNSResourceRecord> > tcache;
       // reap all answers from this packet that are acceptable
@@ -451,7 +454,7 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
 	    
 	    DNSResourceRecord rr=*i;
 	    rr.d_place=DNSResourceRecord::ANSWER;
-	    rr.ttl+=d_now;
+	    rr.ttl+=d_now.tv_sec;
 	    //	  rr.ttl=time(0)+10+10*rr.qtype.getCode();
 	    tcache[toLower(i->qname)+"|"+i->qtype.getName()].insert(rr);
 	  }
@@ -488,13 +491,13 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
 
 	  NegCacheEntry ne;
 	  ne.name=i->qname;
-	  ne.ttd=d_now + i->ttl;
+	  ne.ttd=d_now.tv_sec + i->ttl;
 	  s_negcache[toLower(qname)]=ne;
 	  negindic=true;
 	}
 	else if(i->d_place==DNSResourceRecord::ANSWER && i->qname==qname && i->qtype.getCode()==QType::CNAME && (!(qtype==QType(QType::CNAME)))) {
 	  ret.push_back(*i);
-	  newtarget=i->content;
+	  newtarget=toLowerCanonic(i->content);
 	}
 	// for ANY answers we *must* have an authoritive answer
 	else if(i->d_place==DNSResourceRecord::ANSWER && toLower(i->qname)==toLower(qname) && 
@@ -518,7 +521,7 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
 	  }
 	  else 
 	    LOG<<prefix<<qname<<": got upwards/level NS record '"<<i->qname<<"' -> '"<<i->content<<"', had '"<<auth<<"'"<<endl;
-	  nsset.insert(toLower(i->content));
+	  nsset.insert(toLowerCanonic(i->content));
 	}
 	else if(i->d_place==DNSResourceRecord::AUTHORITY && endsOn(qname,i->qname) && i->qtype.getCode()==QType::SOA && 
 	   d_lwr.d_rcode==RCode::NoError) {
@@ -527,7 +530,7 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
 	  
 	  NegCacheEntry ne;
 	  ne.name=i->qname;
-	  ne.ttd=d_now + i->ttl;
+	  ne.ttd=d_now.tv_sec + i->ttl;
 	  s_negcache[toLower(qname)+"|"+qtype.getName()]=ne;
 	  negindic=true;
 	}
@@ -547,7 +550,6 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
 	return doResolve(newtarget, qtype, ret,0,beenthere2);
       }
       if(nsset.empty() && !d_lwr.d_rcode) {
-
 	LOG<<prefix<<qname<<": status=noerror, other types may exist, but we are done "<<(negindic ? "(have negative SOA)" : "")<<endl;
 	return 0;
       }
@@ -559,7 +561,7 @@ int SyncRes::doResolveAt(set<string> nameservers, string auth, const string &qna
       }
       else {
 	LOG<<prefix<<qname<<": status=NS "<<*tns<<" is lame for '"<<auth<<"', trying sibling NS"<<endl;
-	s_throttle.throttle(d_now, remoteIP+"|"+qname+"|"+qtype.getName(),60,0);
+	s_throttle.throttle(d_now.tv_sec, remoteIP+"|"+qname+"|"+qtype.getName(),60,0);
       }
     }
   }
@@ -583,7 +585,16 @@ void SyncRes::addCruft(const string &qname, vector<DNSResourceRecord>& ret)
        ((k->d_place==DNSResourceRecord::AUTHORITY || k->d_place==DNSResourceRecord::ANSWER) && k->qtype==QType(QType::NS))) {
       LOG<<d_prefix<<qname<<": record '"<<k->content<<"|"<<k->qtype.getName()<<"' needs IP for additional processing"<<endl;
       set<GetBestNSAnswer>beenthere;
-      doResolve(k->content,QType(QType::A),addit,1,beenthere);
+      if(k->qtype==QType(QType::MX)) {
+	string::size_type pos=k->content.find_first_not_of(" \t0123456789"); // chop off the priority
+	if(pos!=string::npos)
+	  doResolve(toLowerCanonic(k->content.substr(pos)), QType(QType::A),addit,1,beenthere);
+	else
+	  doResolve(toLowerCanonic(k->content), QType(QType::A),addit,1,beenthere);
+      }
+      else
+	doResolve(k->content,QType(QType::A),addit,1,beenthere);
+
       if(arg().mustDo("aaaa-additional-processing"))
 	doResolve(k->content,QType(QType::AAAA),addit,1,beenthere);
     }
@@ -606,7 +617,7 @@ void SyncRes::addAuthorityRecords(const string& qname, vector<DNSResourceRecord>
   for(set<DNSResourceRecord>::const_iterator k=bestns.begin();k!=bestns.end();++k) {
     DNSResourceRecord ns=*k;
     ns.d_place=DNSResourceRecord::AUTHORITY;
-    ns.ttl-=d_now;
+    ns.ttl-=d_now.tv_sec;
     ret.push_back(ns);
   }
 }
